module Fluere
  module Worker
    class Base
      def task_list
        raise("abstract")
      end

      def worker_class
        raise("abstract")
      end

      def task_implementations
        raise("abstract")
      end

      def aws_flow_worker
        @aws_flow_worker ||= worker_class.new(
          Fluere.swf.client,
          Fluere.domain,
          task_list,
          *task_implementations
        ) {{
          logger:      Fluere::Config.logger,
          use_forking: Fluere::Config.use_forking
        }}
      end

      def start
        aws_flow_worker.start
      end

      def run_once
        aws_flow_worker.run_once
      end

      def register
        aws_flow_worker.register
      end
    end

    class Workflow < Base
      def task_list
        Fluere.decisions_task_list
      end

      def worker_class
        AWS::Flow::WorkflowWorker
      end

      def task_implementations
        Fluere.workflows
      end
    end

    class Activity < Base
      def task_list
        Fluere.activities_task_list
      end

      def worker_class
        AWS::Flow::ActivityWorker
      end

      def task_implementations
        Fluere.activities
      end
    end
  end
end
